package pl.edu.pg.eti.kask.rpg.datastore.component;

import lombok.extern.java.Log;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import pl.edu.pg.eti.kask.rpg.character.entity.Character;
import pl.edu.pg.eti.kask.rpg.character.entity.Profession;
import pl.edu.pg.eti.kask.rpg.serialization.component.CloningUtility;
import pl.edu.pg.eti.kask.rpg.user.entity.User;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * For the sake of simplification instead of using real database this example is using a data source object which should
 * be put in servlet context in a single instance. In order to avoid {@link java.util.ConcurrentModificationException}
 * all methods are synchronized. Normally synchronization would be carried on by the database server. Caution, this is
 * very inefficient implementation but can be used to present other mechanisms without obscuration example with ORM
 * usage.
 */
@Log
@Component
public class DataStore {

    /**
     * Set of all available professions.
     */
    private final Set<Profession> professions = new HashSet<>();

    /**
     * Set of all characters.
     */
    private final Set<Character> characters = new HashSet<>();

    /**
     * Set of all users.
     */
    private final Set<User> users = new HashSet<>();

    /**
     * Component used for creating deep copies.
     */
    private final CloningUtility cloningUtility;

    /**
     * @param cloningUtility component used for creating deep copies
     */
    @Autowired
    public DataStore(CloningUtility cloningUtility) {
        this.cloningUtility = cloningUtility;
    }

    /**
     * Seeks for all professions.
     *
     * @return list (can be empty) of all professions
     */
    public synchronized List<Profession> findAllProfessions() {
        return professions.stream()
                .map(cloningUtility::clone)
                .collect(Collectors.toList());
    }

    /**
     * Creates new or updates existing profession.
     *
     * @param value user to be updated
     */
    public synchronized void saveProfession(Profession value) {
        professions.removeIf(character -> character.getId().equals(value.getId()));
        professions.add(cloningUtility.clone(value));
    }

    /**
     * Seeks for all characters.
     *
     * @return list (can be empty) of all characters
     */
    public synchronized List<Character> findAllCharacters() {
        return characters.stream()
                .map(cloningUtility::clone)
                .collect(Collectors.toList());
    }

    /**
     * Creates new or updates existing character.
     *
     * @param value character to be updated
     * @throws IllegalArgumentException when {@link User} or {@link Profession} with provided uuid does not exist
     */
    public synchronized void saveCharacter(Character value) throws IllegalArgumentException {
        if (value.getUser() != null) {
            users.stream()
                    .filter(user -> user.getId().equals(value.getUser().getId()))
                    .findFirst()
                    .ifPresentOrElse(
                            value::setUser,
                            () -> {
                                throw new IllegalArgumentException("No user with id \"%s\".".formatted(value.getUser().getId()));
                            }
                    );
        }
        if (value.getProfession() != null) {
            professions.stream()
                    .filter(profession -> profession.getId().equals(value.getProfession().getId()))
                    .findFirst()
                    .ifPresentOrElse(
                            value::setProfession,
                            () -> {
                                throw new IllegalArgumentException("No user with id \"%s\".".formatted(value.getUser().getId()));
                            }
                    );
        }
        characters.removeIf(character -> character.getId().equals(value.getId()));
        characters.add(value);
    }

    /**
     * Deletes existing character.
     *
     * @param id id of character to be deleted
     * @throws IllegalArgumentException if character with provided id does not exist
     */
    public synchronized void deleteCharacter(UUID id) throws IllegalArgumentException {
        if (!characters.removeIf(character -> character.getId().equals(id))) {
            throw new IllegalArgumentException("The character with id \"%s\" does not exist".formatted(id));
        }
    }

    /**
     * Seeks for all users.
     *
     * @return list (can be empty) of all users
     */
    public synchronized List<User> findAllUsers() {
        return users.stream()
                .map(cloningUtility::clone)
                .collect(Collectors.toList());
    }

    /**
     * Creates new or updates existing user.
     *
     * @param value user to be updated
     */
    public synchronized void saveUser(User value) {
        users.removeIf(character -> character.getId().equals(value.getId()));
        users.add(cloningUtility.clone(value));
    }

}
